import sys

# --- Module-level Constant ---
# This remains defined outside the class as it's a fixed value.
C_13523H = 0x13523

class MapGenerator:
    """
    Generates 10x10 grids and corresponding map control states based on
    user ID and solve count, without using global variables for state.

    Each call to generate_map produces an independent result.
    """

    def __init__(self):
        """
        The generator itself is stateless between calls to generate_map.
        State is created and returned by the generate_map method.
        """
        pass # No persistent state needed in the instance itself

    def _add_laser(self, map_control, row_idx):
        """Internal helper: Sets the map control state for a row (1-8) to laser."""
        # Modifies the map_control list passed to it.
        if 1 <= row_idx <= 8: # Check bounds for dynamic part (indices 1-8)
            map_control[row_idx] = 2
        else:
            # This path indicates an internal logic error if reached from generate_map
            print(f"Internal Error: _add_laser called with unexpected row index {row_idx}", file=sys.stderr)

    def _add_wall(self, map_control, grid_state, row_idx, userid, solve_count):
        """Internal helper: Adds a wall pattern to a row (1-8)."""
        # Modifies the map_control and grid_state lists passed to it.
        if not (1 <= row_idx <= 8):
            print(f"Internal Error: _add_wall called with invalid row index {row_idx}. Expected 1-8.", file=sys.stderr)
            return # Avoid modifying incorrect rows

        local_8 = 0
        # Use row_idx directly, as it corresponds to the conceptual 'k' (1-8)
        base_val = userid + solve_count + row_idx

        # Modulo check depends on the row index (1-8)
        if row_idx % 3 == 0:
            local_8 = ((base_val * C_13523H) & 0x3ff) ^ 0x2aa
        elif row_idx % 3 == 1:
            local_8 = ((base_val * C_13523H) & 0x3ff) ^ 0xfffffddd
        else: # row_idx % 3 == 2
            local_8 = 0x288

        # Update the 10 grid cells (columns 0-9) for this row (row_idx)
        start_index = row_idx * 10
        # Check bounds for the flat grid_state list (size 100)
        if 0 <= start_index and start_index + 9 < len(grid_state): # len should be 100
            for i in range(10): # i represents the column index (0-9)
                bit = (local_8 >> i) & 1
                grid_state[start_index + i] = bit
        else:
            # Should not happen if row_idx is 1-8 and grid_state has size 100
            print(f"Internal Warning: _add_wall grid access error for row_idx {row_idx}", file=sys.stderr)

        # Update map control state at the corresponding index (1-8)
        if 1 <= row_idx <= 8:
            map_control[row_idx] = 1


    def _clear_wall(self, map_control, grid_state, row_idx, userid, solve_count):
        """Internal helper: Clears/sets a wall pattern in a row (1-8)."""
        # Modifies the map_control and grid_state lists passed to it.
        if not (1 <= row_idx <= 8):
            print(f"Internal Error: _clear_wall called with invalid row index {row_idx}. Expected 1-8.", file=sys.stderr)
            return

        local_8 = 0
        # Use row_idx directly (corresponds to 'k' 1-8)
        base_val = userid + solve_count + row_idx

        # Modulo check depends on the row index (1-8)
        if row_idx % 3 == 0:
            local_8 = (base_val * C_13523H) & 0x3ff
        elif row_idx % 3 == 1:
            val10 = (base_val * C_13523H) & 0x3ff
            local_8 = val10 ^ 0x3ff # Flip the lower 10 bits
        else: # row_idx % 3 == 2
            local_8 = 0xffffffcf

        # Update the 10 grid cells (columns 0-9) for this row (row_idx)
        start_index = row_idx * 10
        # Check bounds for the flat grid_state list (size 100)
        if 0 <= start_index and start_index + 9 < len(grid_state): # len should be 100
            for i in range(10): # i represents the column index (0-9)
                bit = (local_8 >> i) & 1
                grid_state[start_index + i] = bit
        else:
            # Should not happen if row_idx is 1-8 and grid_state has size 100
            print(f"Internal Warning: _clear_wall grid access error for row_idx {row_idx}", file=sys.stderr)

        # Update map control state at the corresponding index (1-8)
        if 1 <= row_idx <= 8:
            map_control[row_idx] = 0 # Mark row as cleared/empty


    def generate_map(self, userid, solve_count):
        """
        Generates a 10x10 grid and map_control based on userid and solve_count.

        This method is self-contained regarding state generation. It creates
        new lists for map_control and grid_state for each call.

        Args:
            userid (int): The user ID.
            solve_count (int): The solve count.

        Returns:
            tuple: A tuple containing:
                - list: The generated map_control (size 10).
                - list: The generated grid_state (flat list, size 100).
                - int: The value corresponding to DAT_001200a9 (always 3 here).
        """
        # Initialize state locally for this specific generation run
        map_control = [3] + [0] * 8 + [3] # Create new list: [3, 0, ..., 0, 3]
        grid_state = [0] * 100           # Create new zeroed grid list

        # Loop conceptually over k = 1 through 8 (rows 1-8)
        k = 1
        while True:
            if k > 8:
                break # Exit loop instead of return

            # Current grid row index (1-8) being processed
            current_row_idx = k

            # Calculate the determining value uVar1 using 'k'
            shift_amount = (k & 0xf) << 1
            calc_value = (userid + solve_count + k) * C_13523H
            uVar1 = (calc_value >> shift_amount) & 3

            # --- Decision logic based on uVar1 ---
            # map_control index for previous step (0-7)
            prev_map_idx = k - 1

            if uVar1 == 2:
                # Pass the current map_control and grid_state lists to be modified
                self._add_wall(map_control, grid_state, current_row_idx, userid, solve_count)
            elif uVar1 == 0:
                # Check state stored at map_control[k-1] (indices 0-7)
                if map_control[prev_map_idx] != 0:
                     self._clear_wall(map_control, grid_state, current_row_idx, userid, solve_count)
                else:
                     self._add_wall(map_control, grid_state, current_row_idx, userid, solve_count)
            elif uVar1 == 1:
                self._add_wall(map_control, grid_state, current_row_idx, userid, solve_count)
            else: # uVar1 == 3
                self._add_laser(map_control, current_row_idx) # Doesn't need grid_state

            k += 1 # Move to the next 'k' / row index

        # Return the generated state created within this method call
        return map_control, grid_state

def map_transition(map_control, grid, counter):
    grid_new = grid.copy()
    for row in range(1, 9):
        if map_control[row] == 1:
            if row % 2 == 0:
                grid_new = grid_new[:row*10] + grid_new[row*10+1:row*10+10] + [grid_new[row*10]] + grid_new[row*10+10:]
            else:
                grid_new = grid_new[:row*10] + [grid_new[row*10+9]] + grid_new[row*10:row*10+9] + grid_new[row*10+10:]
        elif map_control[row] == 2:
            if counter % 7 == 3:
                grid_new = grid_new[:row*10] + [1]*10 + grid_new[row*10+10:]
            else:
                grid_new = grid_new[:row*10] + [0]*10 + grid_new[row*10+10:]
    return grid_new

from pwn import *

if __name__ == "__main__":
    generator = MapGenerator()

    # io = process(["qemu-aarch64", "-L", "/usr/aarch64-linux-gnu/", "segvroad"])
    io = remote("segvroad.atreides.b01lersc.tf", 8443, ssl=True)
    io.recvline()
    userid = int(io.recvline().split(b": ")[1][:-1])
    
    counter = 1
    for solve_count in range(10):
        print(f"--- Generating Map for User {userid}, Solve Count {solve_count} ---")
        map_control, grid_state = generator.generate_map(userid, solve_count)

        grid_states = []
        grid_states.append(grid_state)

        print("Map Control 1:", map_control)
        print("Grid State 1 (10x10):")
        for r in range(10):
            start = r * 10
            end = start + 10
            row_data = grid_state[start:end]
            map_state = map_control[r]
            print(f"  Row {r}: {row_data} (map_control[{r}]: {map_state})")
        print("-" * 30)

        for i in range(1, 50):
            grid_state = map_transition(map_control, grid_state, i)
            grid_states.append(grid_state)
        
        queue = []
        visited = [[False for _ in range(100)] for _ in range(50)]
        queue.append(('', 0, 0, 0)) # (path, x, y, t)
        sol = ''
        while len(queue) != 0:        
            p, x, y, t = queue.pop()
            visited[t][x + y*10] = True
            if x == 9 and y == 9:
                sol = p
                queue = []
                break
            if t == 35:
                continue

            g = grid_states[t+1] # next possible state

            if not visited[t+1][min(x+1, 9) + y*10]:
                visited[t+1][min(x+1, 9) + y*10] = True
                if g[min(x+1, 9) + y*10] == 0:
                    queue.append((p + 'd', min(x+1, 9), y, t+1))
            
            if not visited[t+1][max(x-1, 0) + y*10]:
                visited[t+1][max(x-1, 0) + y*10] = True
                if g[max(x-1, 0) + y*10] == 0:
                    queue.append((p + 'a', max(x-1, 0), y, t+1))

            if not visited[t+1][x + min((y+1), 9)*10]:
                visited[t+1][x + min((y+1), 9)*10] = True
                if g[x + min((y+1), 9)*10] == 0:
                    queue.append((p + "w", x, min(y+1, 9), t+1))

            if not visited[t+1][x + max(y-1, 0)*10]:
                visited[t+1][x + max(y-1, 0)*10] = True
                if g[x + max(y-1, 0)*10] == 0:
                    queue.append((p + "s", x, max(y-1, 0), t+1))

        print(f"Solution for Stage {solve_count}: {sol}")
        for c in sol:
            io.sendline(c.encode())
        counter += len(sol)
    
    print(io.recvall())



"""
0x00    0x00    0x00    0x00    0x00    0x00    0x00    0x00    0x00    0x00    
0x00    0x00    0x01    0x00    0x00    0x01    0x00    0x00    0x01    0x01    
0x00    0x00    0x00    0x00    0x00    0x00    0x00    0x00    0x00    0x00    
0x01    0x00    0x00    0x00    0x00    0x00    0x01    0x01    0x01    0x01
0x00    0x01    0x00    0x00    0x00    0x00    0x00    0x01    0x01    0x01    
0x00    0x01    0x00    0x00    0x00    0x01    0x00    0x00    0x00    0x01    
0x01    0x01    0x00    0x00    0x00    0x00    0x01    0x01    0x00    0x01    
0x00    0x00    0x00    0x00    0x00    0x00    0x00    0x00    0x00    0x00
0x01    0x01    0x01    0x01    0x00    0x00    0x01    0x01    0x01    0x01    
0x00    0x00    0x00    0x00    0x00    0x00    0x00    0x00    0x00    0x00

"""